@@ -23,11 +23,10 @@
 
 #include <config.h>
 
-#include "gnome-i18nP.h"
+#include <glib/gi18n-lib.h>
 
 #include "gnome-exec.h"
 #include "gnome-util.h"
-#include "gnome-i18n.h"
 #include "gnome-gconfP.h"
 #include "gnome-init.h"
 
@@ -44,7 +43,6 @@
 
 #include <gconf/gconf-client.h>
 
-#include <popt.h>
 
 #include <errno.h>
 #ifndef errno
@@ -403,7 +401,7 @@ gnome_prepend_terminal_to_vector (int *argc, char ***argv)
 
 	client = gconf_client_get_default ();
 	terminal = gconf_client_get_string (client, "/desktop/gnome/applications/terminal/exec", NULL);
-	g_object_unref (G_OBJECT (client));
+	g_object_unref (client);
 	
 	if (terminal) {
 		gchar *command_line;
@@ -424,11 +422,6 @@ gnome_prepend_terminal_to_vector (int *argc, char ***argv)
 		g_free (command_line);
 		g_free (exec_flag);
 		g_free (terminal);
-#if 0
-	    poptParseArgvString (terminal, &term_argc, &temp_argv);
-	    term_argv = g_strdupv ((gchar **) temp_argv);
-	    g_free (terminal);
-#endif
 	}
 
 	if (term_argv == NULL) {
