@@ -38,7 +38,7 @@
 #include <glib.h>
 #include <glib/goption.h>
 #include <glib/gstdio.h>
-#include <glib/gi18n-lib.h>
+#include <libgnome/gnome-i18n.h>
 
 #include "gnome-init.h"
 #include "gnome-gconfP.h"
@@ -339,6 +339,8 @@ safe_mkdir (const char *pathname, mode_t mode)
 static void
 libgnome_userdir_setup (gboolean create_dirs)
 {
+	struct stat statbuf;
+	
 	if(!gnome_user_dir) {
                 const char *override;
 
@@ -355,11 +357,7 @@ libgnome_userdir_setup (gboolean create_dirs)
 
                         /* chop trailing slash */
                         len = strlen (gnome_user_dir);
-                        if (len > 1 && (gnome_user_dir[len-1] == G_DIR_SEPARATOR
-#ifdef G_OS_WIN32
-					|| gnome_user_dir[len-1] == '/'
-#endif
-					))
+                        if (len > 1 && G_IS_DIR_SEPARATOR (gnome_user_dir[len - 1]))
                                 gnome_user_dir[len - 1] = '\0';
 
                         gnome_user_private_dir = g_strconcat (gnome_user_dir,
@@ -402,8 +400,15 @@ libgnome_userdir_setup (gboolean create_dirs)
 	}
 
 
+       if (g_stat (gnome_user_private_dir, &statbuf) < 0) {
+               g_printerr (_("Could not stat private per-user gnome configuration directory `%s': %s\n"),
+                       gnome_user_private_dir, strerror(errno));
+               exit(1);
+       }
+
 	/* change mode to 0700 on the private directory */
-	if (g_chmod (gnome_user_private_dir, 0700) < 0) {
+	if (((statbuf.st_mode & 0700) != 0700 )  &&
+            g_chmod (gnome_user_private_dir, 0700) < 0) {
 		g_printerr (_("Could not set mode 0700 on private per-user gnome configuration directory `%s': %s\n"),
 			gnome_user_private_dir, strerror(errno));
 		exit(1);
